#!/usr/bin/python -u
# -*- mode:python; coding:utf-8; tab-width:4 -*-

import sys
import Ice
Ice.loadSlice('-I {} cannon.ice'.format(Ice.getSliceDir()))
Ice.loadSlice('-I {} container.ice'.format(Ice.getSliceDir()))
import Cannon
import math
import Services
from matrix_utils import (matrix_horizontal_shift,
                          matrix_vertical_shift,
                          matrix_split, matrix_join)


class FrontendI(Cannon.Frontend):
    def __init__(self, processors):
        self.processors=processors
		self.nProcessors=len(self.processors)
		self.order = int(math.sqrt(self.nProcessors))
		self.collector=None
		self.collectorprx=None
	
	def CambiarCollector(self, collector):
		self.proxy=collector
	
	def crearCollector(self):
		self.collector = CollectorI(self.order)
    	self.collectorprx = Cannon.CollectorPrx.checkedCast(self.adapter.addWithUUID(self.collector))
		

	def borrarCollector(self):
		self.adapter.remove(self.collectorprx.ice_getIdentity())
        del self.collector

	def obtenerResultado(self):
		resultadoRecibido=1
		while resultadoRecibido:
			resultado=self.collector.get_result()
			if not(resultado == None):
				resultadoRecibido=0
		return resultado


	def multiply(self, A, B, current=None):
		self.adapter = current.adapter
        self.crearCollector()

		self.init_processors()
        self.load_processors(A, B)
		
		resultado = self.obtenerResultado()

		self.borrarCollector()
		
        return resultado

    def init_processors(self):
		
		cont=-1
		for i in range(0, self.order):
			for j in range(0, self.order):
				cont=cont+1
				above = ((i+self.order-1)%self.order)*self.order + j
				left = i*self.order + (j+self.order-1)%self.order
				self.processors[i*self.order+j].init(cont, self.order, self.processors[above], self.processors[left],  self.collectorprx)

    def load_processors(self, A, B):
		orderBloque = int(A.ncols/self.order)
		
		matrizDesplazadaA = matrix_horizontal_shift(A, orderBloque)
		bloquesA = matrix_split(matrizDesplazadaA, orderBloque)
		
		matrizDesplazadaB = matrix_vertical_shift(B, orderBloque)
		bloquesB = matrix_split(matrizDesplazadaB, orderBloque)

		for i in range(0, self.nProcessors):
			self.processors[i].injectA(bloquesA[i],0)
			self.processors[i].injectB(bloquesB[i],0)


class CollectorI(Cannon.Collector):
    def __init__(self, order):
		self.bloques = []; self.nBloques = 0;
        self.g_order = order
        
        for i in range(order ** 2):
        	self.bloques.append(None)


    def inject(self, index, block, current=None):
		self.bloques[index] = block
		self.nBloques+=1


	def get_result(self):
    	if(self.nBloques == self.g_order ** 2):
        	return matrix_join(*self.bloques)
        else:
        	return None

class Server(Ice.Application):
    def run(self, args):
        processors = []
        broker = self.communicator()
        
        proxy_container = broker.stringToProxy(args[1])
        
        print("Esperando container...")
        
        while 1:
        	try:
        		proxy_container.ice_ping()
        		print("...Done"); break
        	except Ice.NoEndpointException:
        		time.sleep(1)
        		
        container = Services.ContainerPrx.checkedCast(proxy_container)
        
        if not container:
			raise RuntimeError('Invalid proxy')


		prox_procesadores = list(container.list().values())
        
        for i in range(len(prox_procesadores)):
        	processors.append(Cannon.ProcessorPrx.checkedCast(prox_procesadores[i]))
        print('found {} processors'.format(len(processors)))

        servant = FrontendI(processors)
	
        adapter = broker.createObjectAdapter('FrontendAdapter')
        proxy = adapter.add(servant, broker.stringToIdentity("frontend"))

        print(proxy)

        adapter.activate()
        self.shutdownOnInterrupt()
        broker.waitForShutdown()


if __name__ == '__main__':
    sys.exit(Server().main(sys.argv))
